@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2016  B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2017  B.A.T.M.A.N. contributors:
  *
  * Andreas Langer <an.langer@gmx.de>, Marek Lindner <mareklindner@neomailbox.ch>
  *
@@ -45,7 +45,7 @@
 #include "icmp_helper.h"
 
 
-char is_aborted = 0;
+static volatile sig_atomic_t is_aborted = 0;
 
 
 static void ping_usage(void)
@@ -133,6 +133,8 @@ int ping(char *mesh_iface, int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
+	check_root_or_die("batctl ping");
+
 	dst_string = argv[found_args];
 	bat_hosts_init(0);
 	bat_host = bat_hosts_find_by_name(dst_string);
