@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2016  B.A.T.M.A.N. contributors:
+ * Copyright (C) 2009-2017  B.A.T.M.A.N. contributors:
  *
  * Marek Lindner <mareklindner@neomailbox.ch>
  *
@@ -156,6 +156,8 @@ int handle_loglevel(char *mesh_iface, int argc, char **argv)
 	snprintf(path_buff, PATH_BUFF_LEN, SYS_BATIF_PATH_FMT, mesh_iface);
 
 	if (argc != 1) {
+		check_root_or_die("batctl loglevel");
+
 		for (i = 1; i < argc; i++) {
 			if (strcmp(argv[i], "none") == 0) {
 				log_level = 0;
@@ -268,6 +270,8 @@ int handle_sys_setting(char *mesh_iface, int setting, int argc, char **argv)
 		goto out;
 	}
 
+	check_root_or_die("batctl");
+
 	if (!batctl_settings[setting].params)
 		goto write_file;
 
@@ -380,6 +384,8 @@ int handle_gw_setting(char *mesh_iface, int argc, char **argv)
 		goto out;
 	}
 
+	check_root_or_die("batctl gw_mode");
+
 	if (strcmp(argv[1], "client") == 0)
 		gw_mode = GW_MODE_CLIENT;
 	else if (strcmp(argv[1], "server") == 0)
@@ -449,6 +455,8 @@ int handle_ra_setting(int argc, char **argv)
 		}
 	}
 
+	check_root_or_die("batctl routing_algo");
+
 	if (argc == 2) {
 		res = write_file(SYS_SELECTED_RA_PATH, "", argv[1], NULL);
 		goto out;
